import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import IconButton from '@sinoui/core/IconButton';
import Popper from '@sinoui/core/Popper';
import useClickAwayListener from '@sinoui/core/useClickAwayListener';
import React, { useRef, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import styled from 'styled-components';
import { EDITOR_CHANGE, eventbus } from '../../constants';
import setCellTextStyleHelper from '../../utils/setCellTextStyleHelper';
import SketchPicker from '../color-picker';
import useListenEvent from '../useEventListener';
const StyledIconButton = styled(IconButton) `
  flex-direction: column;
  border-radius: 0;
`;
const ColorWrapper = styled.div `
  background: #fff;
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.1);
  cursor: pointer;
  width: 20px;
`;
const ColorPanel = styled.div `
  width: 100%;
  height: 4px;
  background: ${({ color }) => color};
`;
const PickerPanelWrapper = styled.div `
  position: absolute;
  z-index: 2;
`;
/**
 * 文本颜色选择器
 *
 * @param root0
 * @param root0.editor
 * @param root0.selectedRange
 * @param root0.editingCell
 * @returns
 */
function ColorPickerButton({ editor = {}, selectedRange, editingCell, }) {
    const anchorEl = useRef(null);
    const popperRef = useRef(null);
    const [open, setOpen] = useState(false);
    const [color, setColor] = useState('#F17013');
    const dispatch = useDispatch();
    const rows = useSelector((state) => state.present.rows);
    const merges = useSelector((state) => state.present.merges);
    const handleMouseDown = (event) => {
        event.preventDefault();
        setOpen(true);
    };
    useListenEvent(eventbus, EDITOR_CHANGE, () => { });
    useClickAwayListener([anchorEl, popperRef], () => setOpen(false));
    const handleChange = (colorInfo) => {
        setColor(colorInfo.hex);
        setCellTextStyleHelper(editor, dispatch, 'color', colorInfo.hex, rows, merges, selectedRange, editingCell);
    };
    return (_jsxs(_Fragment, { children: [_jsxs(StyledIconButton, { dense: true, ref: anchorEl, onMouseDown: handleMouseDown, children: [_jsx("div", { children: "A" }), _jsx(ColorWrapper, { children: _jsx(ColorPanel, { color: color }) })] }), _jsx(Popper, { referenceElement: anchorEl, open: open, placement: "bottom-start", ref: popperRef, children: _jsx(PickerPanelWrapper, { children: _jsx(SketchPicker, { color: color, onChange: handleChange }) }) })] }));
}
export default React.memo(ColorPickerButton);
