import { jsx as _jsx } from "react/jsx-runtime";
import appSetting from '@sinoform/app-setting';
import React, { useState } from 'react';
import { useDispatch, useSelector, useStore } from 'react-redux';
import { EDITOR_CHANGE, eventbus } from '../../constants';
import CustomEditor from '../../utils/CustomEditor';
import getCellValue from '../../utils/getCellValue';
import setCellTextStyleHelper from '../../utils/setCellTextStyleHelper';
import useWatchProp from '../../utils/useWatchProp';
import useListenEvent from '../useEventListener';
/**
 * 获取选中区域的字体类型
 * @param state
 * @param selectedRange
 * @param initValue
 * @returns
 */
const getSelectedFontFamily = (state, selectedRange, initValue) => {
    const currentCellValue = getCellValue(state, selectedRange[0][0], selectedRange[0][1]);
    const isAllHasStyle = currentCellValue.every((element) => element.children.every((textItem) => textItem.fontFamily));
    return isAllHasStyle
        ? currentCellValue[0].children[0].fontFamily
        : initValue;
};
const FONT_FAMILY_ITEMS = appSetting.printOutConfig.fonts.map((font) => font.title);
/**
 * 字体选择组件
 *
 * @param props
 * @returns
 */
function FontFamilySelectButton(props) {
    const { editor = {}, selectedRange, editingCell } = props;
    const initValue = '仿宋_GB2312';
    const [value, setValue] = useState(initValue);
    const dispatch = useDispatch();
    const state = useStore().getState().present;
    const rows = useSelector((reduxState) => reduxState.present.rows);
    const merges = useSelector((reduxState) => reduxState.present.merges);
    useWatchProp(selectedRange, () => {
        if (selectedRange) {
            setValue(getSelectedFontFamily(state, selectedRange, initValue));
        }
    });
    useListenEvent(eventbus, EDITOR_CHANGE, (currentEditor) => {
        setValue(CustomEditor.getFontFamily(currentEditor) ?? initValue);
    });
    const handleChange = (event) => {
        setValue(event.target.value);
        setCellTextStyleHelper(editor, dispatch, 'fontFamily', event.target.value, rows, merges, selectedRange, editingCell);
    };
    return (_jsx("select", { value: value, onChange: handleChange, children: FONT_FAMILY_ITEMS.map((family) => (_jsx("option", { value: family, children: family }, family))) }));
}
export default React.memo(FontFamilySelectButton);
