import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
/* eslint-disable react/no-array-index-key */
/* eslint-disable no-nested-ternary */
import { useCallback, useEffect, useMemo, useRef } from 'react';
import classnames from 'classnames';
import { useSelector, useDispatch } from 'react-redux';
import styled from 'styled-components';
import { DEFAULT_CELL_HEIGHT } from '@sinoform/print-out-core';
import { changeRowHeight } from '../reducer';
import Td from './Td';
import { checkTdIsMerged, getCellBorderStyle, getCellHeight, getCellWidth, transformCMToPxByConfig, } from './util';
import { MIN_CELL_HEIGHT } from '../constants';
import drawRange from './drawRange';
const MarkLine = styled.div `
  position: absolute;

  user-select: none;
`;
const SelectRange = styled.div `
  position: absolute;
  display: none;
  border: 2px solid #4b89ff;
  user-select: none;
  pointer-events: none;
  box-sizing: border-box;
`;
export default function TBody({ selectedRange, setEditor, handleTdDoubleClick, fields, }) {
    const printConfig = useSelector((state) => state.present);
    const rowHeight = useSelector((state) => {
        const { rows } = state.present;
        let sum = 0;
        for (let i = 0; i < rows.len; i += 1) {
            sum += rows[i]?.height ?? DEFAULT_CELL_HEIGHT;
        }
        return sum;
    });
    const colLenArray = Array.from({ length: printConfig.cols.len });
    const rowLenArray = Array.from({ length: printConfig.rows.len });
    const dragTrIdxRef = useRef();
    const tBodyRef = useRef(null);
    const horizontalMarkLineRef = useRef(null);
    const markLineOffsetTop = useMemo(() => transformCMToPxByConfig(printConfig.config, 'h'), [printConfig.config]);
    const drawMarkLine = useCallback(() => {
        if (!horizontalMarkLineRef.current)
            return;
        const trs = tBodyRef.current.querySelectorAll('tr');
        let sum = 0;
        for (let i = 0; i < trs.length; i += 1) {
            if (trs[i].offsetTop + trs[i].offsetHeight <= markLineOffsetTop) {
                sum = trs[i].offsetTop + trs[i].offsetHeight;
            }
            else {
                break;
            }
        }
        horizontalMarkLineRef.current.style.top = `${sum}px`;
        horizontalMarkLineRef.current.style.left = `0px`;
        horizontalMarkLineRef.current.style.width = '100%';
        horizontalMarkLineRef.current.style.height = '1px';
        horizontalMarkLineRef.current.style.borderTop = '1px solid #4b89ff';
    }, [markLineOffsetTop]);
    useEffect(() => {
        drawMarkLine();
    }, [drawMarkLine]);
    useEffect(() => {
        if (selectedRange) {
            drawRange(selectedRange[0], selectedRange[1]);
        }
    }, [selectedRange, rowHeight]);
    const dispatch = useDispatch();
    const isRowInSelectedRange = (rowIdx) => {
        if (!selectedRange)
            return false;
        return selectedRange[0][0] <= rowIdx && rowIdx <= selectedRange[1][0];
    };
    const handleMouseMoveInDocument = (event) => {
        const bottomLine = document.querySelector('.sinoui-drag-bottom-line');
        const theadElement = document.querySelector('thead');
        const { top } = theadElement.getBoundingClientRect();
        if (bottomLine && bottomLine.style.display === 'block') {
            bottomLine.style.top = `${event.clientY - top}px`;
        }
    };
    const handleMouseUpInDocument = (event) => {
        const topLine = document.querySelector('.sinoui-drag-top-line');
        const bottomLine = document.querySelector('.sinoui-drag-bottom-line');
        if (dragTrIdxRef.current &&
            topLine &&
            topLine.style.display === 'block' &&
            bottomLine &&
            bottomLine.style.display === 'block') {
            topLine.style.display = 'none';
            bottomLine.style.display = 'none';
            const theadElement = document.querySelector('thead');
            const { top } = theadElement.getBoundingClientRect();
            const dragCell = document.querySelectorAll('tr')[dragTrIdxRef.current];
            const height = event.clientY - dragCell.offsetTop - top > 0
                ? event.clientY - dragCell.offsetTop - top >= MIN_CELL_HEIGHT
                    ? event.clientY - dragCell.offsetTop - top
                    : MIN_CELL_HEIGHT
                : MIN_CELL_HEIGHT;
            dispatch(changeRowHeight(dragTrIdxRef.current - 1, height));
        }
        document.removeEventListener('mousemove', handleMouseMoveInDocument, false);
        document.removeEventListener('mouseup', handleMouseUpInDocument, false);
    };
    const handleMouseMoveInFirstCol = (event) => {
        const cell = event.target;
        const theadElement = document.querySelector('thead');
        const { top } = theadElement.getBoundingClientRect();
        if (event.clientY - top >= cell.offsetHeight + cell.offsetTop - 10 &&
            cell.tagName === 'TD' &&
            cell.cellIndex === 0) {
            // 设置鼠标样式
            cell.style.cursor = 'row-resize';
        }
        else {
            cell.style.cursor = 'default';
        }
    };
    const handleMouseDownInFirstCol = (event) => {
        const cell = event.target;
        const theadElement = document.querySelector('thead');
        const { top } = theadElement.getBoundingClientRect();
        const topLine = document.querySelector('.sinoui-drag-top-line');
        const bottomLine = document.querySelector('.sinoui-drag-bottom-line');
        if (event.clientY - top >= cell.offsetHeight + cell.offsetTop - 10 &&
            cell.tagName === 'TD' &&
            cell.cellIndex === 0) {
            dragTrIdxRef.current = event.currentTarget.rowIndex;
            // 显示辅助线
            if (topLine && topLine.style.display !== 'block') {
                topLine.style.display = 'block';
                topLine.style.top = `${cell.offsetTop}px`;
            }
            if (bottomLine && bottomLine.style.display !== 'block') {
                bottomLine.style.display = 'block';
                bottomLine.style.top = `${cell.offsetTop + cell.offsetHeight}px`;
            }
            document.addEventListener('mousemove', handleMouseMoveInDocument, false);
            document.addEventListener('mouseup', handleMouseUpInDocument, false);
        }
    };
    return (_jsxs(_Fragment, { children: [_jsx("tbody", { ref: tBodyRef, children: rowLenArray.map((_, rowIdx) => {
                    const row = printConfig.rows[rowIdx];
                    const cells = row?.cells;
                    return (_jsxs("tr", { style: { height: `${row?.height ?? DEFAULT_CELL_HEIGHT}px` }, onMouseMove: handleMouseMoveInFirstCol, onMouseDown: handleMouseDownInFirstCol, children: [_jsx("td", { className: classnames('custom-print-table-col-header', {
                                    'row-in-selected-range': isRowInSelectedRange(rowIdx),
                                }), id: `table-row-header-${rowIdx}`, "data-rowid": rowIdx, children: rowIdx + 1 }), colLenArray.map((_item, colIdx) => {
                                const cell = cells?.[colIdx];
                                if (checkTdIsMerged(rowIdx, colIdx, printConfig.merges)) {
                                    return null;
                                }
                                const { border, ...style } = cell?.style ?? {};
                                return (_jsx(Td, { cell: cell, colIdx: colIdx, rowIdx: rowIdx, handleTdDoubleClick: handleTdDoubleClick, setEditor: setEditor, height: getCellHeight(printConfig, rowIdx, cell), width: getCellWidth(printConfig, colIdx, cell), fields: fields, style: {
                                        ...style,
                                        ...getCellBorderStyle(border),
                                    } }, `${rowIdx}-${colIdx}`));
                            })] }, rowIdx));
                }) }), _jsx(MarkLine, { ref: horizontalMarkLineRef, "data-markline": true }), _jsx(SelectRange, { className: "sinoui-table-select-range" })] }));
}
