/**
 * 计算选中单元格的区域
 *
 * 如果选中区域存在合并的单元格区域超出了选中的边界，需要将选中区域拓展到新的边界，
 * 如果拓展后的边界重新触发了该规则，则递归进行计算
 *
 * @param startPos 选中区域起始位置
 * @param endPos   选中区域结束位置
 * @param merges   所有单元格的合并信息
 */
export default function calcCellRange(startPos, endPos, merges = []) {
    // 按照左上至右下的顺序重新生成开始和结束节点
    const start = [
        Math.min(startPos[0], endPos[0]),
        Math.min(startPos[1], endPos[1]),
    ];
    const end = [
        Math.max(startPos[0], endPos[0]),
        Math.max(startPos[1], endPos[1]),
    ];
    // 判断这个矩形区域内有没有超出边界的合并单元格，如果存在则将单元格的边界拓展到最边缘的位置
    for (let i = start[0]; i <= end[0]; i += 1) {
        for (let j = start[1]; j <= end[1]; j += 1) {
            const merge = merges.find(([[sri, sci], [eri, eci]]) => sri <= i && eri >= i && sci <= j && eci >= j);
            if (merge &&
                (merge[0][0] < start[0] ||
                    merge[0][1] < start[1] ||
                    merge[1][0] > end[0] ||
                    merge[1][1] > end[1])) {
                const newStart = [
                    Math.min(merge[0][0], start[0]),
                    Math.min(merge[0][1], start[1]),
                ];
                const newEnd = [
                    Math.max(merge[1][0], end[0]),
                    Math.max(merge[1][1], end[1]),
                ];
                return calcCellRange(newStart, newEnd, merges);
            }
        }
    }
    return [start, end];
}
