import { Transforms } from 'slate';
import getNodeInfoByPoint from './getNodeInfoByPoint';
import isCodeType from './isCodeType';
/**
 * 自动选中代码块
 *
 * @param editor 编辑器状态
 */
export default function autoSelectCodeBlock(editor) {
    if (!editor || !editor.selection) {
        return;
    }
    const { selection: { anchor, focus }, } = editor;
    const anchorRow = anchor.path[0];
    const anchorCol = anchor.path[1];
    const focusRow = focus.path[0];
    const focusCol = focus.path[1];
    const anchorInfo = getNodeInfoByPoint(editor, anchor);
    const focusInfo = getNodeInfoByPoint(editor, focus);
    if (isCodeType(editor, anchor)) {
        // 从前往后选择，锚点所在元素必须有选中
        if ((anchorRow < focusRow ||
            anchorCol < focusCol ||
            anchor.offset < focus.offset) &&
            anchor.offset < anchorInfo.end) {
            // 如果锚点不是代码块的起点，焦点是代码块且不是代码块的末尾，则自动选中开始代码块和结束代码块
            if (anchor.offset !== 0 &&
                isCodeType(editor, focus) &&
                focus.offset !== focusInfo.end &&
                focus.offset > focusInfo.start) {
                const newSelect = {
                    anchor: { ...anchor, offset: anchorInfo.start },
                    focus: { ...focus, offset: focusInfo.end },
                };
                Transforms.select(editor, newSelect);
                return;
            }
            // 如果锚点不是代码块起点，焦点为任意元素，则自动选中开始代码块
            if (anchor.offset !== 0) {
                const newSelect = {
                    anchor: { ...anchor, offset: anchorInfo.start },
                    focus,
                };
                Transforms.select(editor, newSelect);
                return;
            }
            // 如果焦点所在元素是代码块，且不是代码块的末端，则自动选中结束代码块
            if (isCodeType(editor, focus) && focus.offset !== focusInfo.end) {
                const newSelect = {
                    anchor,
                    focus: { ...focus, offset: focusInfo.end },
                };
                Transforms.select(editor, newSelect);
            }
        }
        else if (
        // 从后往前选择,且必须有元素选中
        (anchorRow > focusRow ||
            anchorCol > focusCol ||
            anchor.offset > focus.offset) &&
            focus.offset < anchorInfo.end &&
            anchor.offset > anchorInfo.start) {
            // 如果锚点不是代码块的末尾，焦点所在元素为代码块且不是代码块的开始,则自动选中锚点和焦点所在的代码块
            if (anchor.offset !== anchorInfo.end &&
                isCodeType(editor, focus) &&
                focus.offset !== focusInfo.start) {
                const newSelect = {
                    anchor: { ...anchor, offset: anchorInfo.end },
                    focus: { ...focus, offset: focusInfo.start },
                };
                Transforms.select(editor, newSelect);
                return;
            }
            // 如果锚点不是所在代码块的末尾，焦点为普通元素，则自动选中锚点所在代码块
            if (anchor.offset !== anchorInfo.end) {
                const newSelect = {
                    anchor: { ...anchor, offset: anchorInfo.end },
                    focus,
                };
                Transforms.select(editor, newSelect);
                return;
            }
            // 如果焦点所在元素为代码块，且焦点所在位置不是代码块的起点，则自动选中焦点所在代码块
            if (isCodeType(editor, focus) && focus.offset !== focusInfo.start) {
                const newSelect = {
                    anchor,
                    focus: { ...focus, offset: focusInfo.start },
                };
                Transforms.select(editor, newSelect);
            }
        }
    }
    else if (isCodeType(editor, focus)) {
        // 从前往后选择，锚点为任意元素，焦点为表达式且未全部选中，则自动选中焦点代码块
        if ((anchorRow < focusRow ||
            anchorCol < focusCol ||
            anchor.offset < focus.offset) &&
            focus.offset > focusInfo.start &&
            focus.offset !== focusInfo.end) {
            const newSelect = {
                anchor,
                focus: { ...focus, offset: focusInfo.end },
            };
            Transforms.select(editor, newSelect);
            return;
        }
        // 从后往前选择，焦点所在位置不是代码块的开始，则自动选中焦点所在代码块
        if ((anchorRow > focusRow ||
            anchorCol > focusCol ||
            anchor.offset > focus.offset) &&
            focus.offset !== focusInfo.start &&
            focus.offset < focusInfo.end) {
            const newSelect = {
                anchor,
                focus: { ...focus, offset: focusInfo.start },
            };
            Transforms.select(editor, newSelect);
        }
    }
}
