import { DEFAULT_CELL_VALUE } from '@sinoform/print-out-core';
import { ActionType } from '../reducer';
import CustomEditor from './CustomEditor';
import getAllSelectedCells from './getAllSelectedCells';
/**
 * 设置单元格文本样式的辅助函数
 *
 * @param editor 编辑器
 * @param dispatch
 * @param styleType 样式类型
 * @param styleValue 样式的值
 * @param rows 行数据
 * @param rows.len
 * @param merges 合并单元格集合
 * @param selectedRange 选中范围
 * @param editingCell 处于编辑状态的单元格
 * @returns
 */
export default function setCellTextStyleHelper(editor, dispatch, styleType, styleValue, rows, merges = [], selectedRange, editingCell) {
    if (!selectedRange) {
        return;
    }
    if (editingCell) {
        switch (styleType) {
            case 'bold':
                CustomEditor.toggleBoldMark(editor);
                break;
            case 'underLine':
                CustomEditor.toggleUnderLineMark(editor);
                break;
            case 'deleteLine':
                CustomEditor.toggleDeleteLineMark(editor);
                break;
            case 'italic':
                CustomEditor.toggleItalicMark(editor);
                break;
            case 'color':
                CustomEditor.changeTextColor(editor, styleValue);
                break;
            case 'fontSize':
                CustomEditor.toggeleFontSize(editor, styleValue);
                break;
            case 'fontFamily':
                CustomEditor.toggeleFontFamily(editor, styleValue);
                break;
            default:
                break;
        }
    }
    else {
        const selectedCells = getAllSelectedCells(selectedRange, merges);
        const items = selectedCells.map(([rowIdx, colIdx]) => {
            const cellValue = rows[rowIdx]?.cells[colIdx]?.value ?? DEFAULT_CELL_VALUE;
            return cellValue.map((item) => ({
                ...item,
                children: item.children.map((textItem) => ({
                    ...textItem,
                    [styleType]: styleType === 'color' ||
                        styleType === 'fontSize' ||
                        styleType === 'fontFamily'
                        ? styleValue
                        : !textItem[styleType],
                })),
            }));
        });
        dispatch({
            type: ActionType.SET_CELLS_TEXT_STYLE,
            payload: {
                cells: selectedCells,
                items,
            },
        });
    }
}
