import globalUI from '@sinoform/global-ui';
import { Transforms } from 'slate';
import checkCellMergedInRow from '../reducer/helper/checkCellMergedInRow';
import { getAllCodeTokensInRow, getAllSubFieldFromToken, } from '../reducer/helper/checkSubFieldInRow';
/**
 * 校验拖拽时子表单是否符合规范
 * @param state
 * @param rowIdx
 * @param colIdx
 * @param cellValue
 * @param editor
 * @param fields
 * @returns
 */
export default function validateSubForm(state, rowIdx, colIdx, cellValue, editor, fields) {
    const tokens = getAllCodeTokensInRow(state, [rowIdx, colIdx], cellValue);
    const subFields = getAllSubFieldFromToken(tokens, fields);
    // 公式涉及到子表单的校验
    // subFields.size > 1: 同行出现不同子表单的表单项
    // checkCellMergedInRow(rowIdx, state.merges): 子表单项所在行存在合并单元格的合并行超出本行
    if (subFields.size > 0) {
        const flag = subFields.size > 1 || checkCellMergedInRow(rowIdx, state.merges);
        if (flag) {
            const text = subFields.size > 1
                ? '同一行单元格的表达式中不能出现不同子表单的表单项'
                : '子表单项所在行存在合并单元格的合并行超出当前行';
            globalUI.message.error(text);
            // 如果存在冲突将焦点重置，否则焦点位置信息错误重新编辑可能会报错
            const selectRange = {
                anchor: {
                    path: [0, 0],
                    offset: 0,
                },
                focus: { path: [0, 0], offset: 0 },
            };
            Transforms.setSelection(editor, selectRange);
            return false;
        }
    }
    return true;
}
