"use strict";

var _interopRequireDefault = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireDefault").default;
var _interopRequireWildcard = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireWildcard").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _objectSpread2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/objectSpread2"));
var _createForOfIteratorHelper2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/createForOfIteratorHelper"));
var _slicedToArray2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/slicedToArray"));
var _objectWithoutProperties2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/objectWithoutProperties"));
var _jsxRuntime = /*#__PURE__*/require("react/jsx-runtime");
var _react = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("react"));
var _slate = /*#__PURE__*/require("slate");
var _slateReact = /*#__PURE__*/require("slate-react");
var _helperFormulaAnalysis = /*#__PURE__*/require("@sinoform/helper-formula-analysis");
var _styledComponents = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("styled-components"));
var _reactRedux = /*#__PURE__*/require("react-redux");
var _isEqual = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("lodash-es/isEqual"));
var _printOutCore = /*#__PURE__*/require("@sinoform/print-out-core");
var _Leaf = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("./Leaf"));
var _autoSelectCodeBlock = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("../../utils/autoSelectCodeBlock"));
var _constants = /*#__PURE__*/require("../../constants");
var _isCodeType = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("../../utils/isCodeType"));
var _useWatchProp = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("../../utils/useWatchProp"));
var _getCellValue = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("../../utils/getCellValue"));
var _excluded = ["value", "bgColor", "width", "fields", "rowIdx", "colIdx", "editor", "readOnly"];
var EditableLayout = /*#__PURE__*/(0, _styledComponents.default)(_slateReact.Editable).withConfig({
  displayName: "SlateEditor__EditableLayout",
  componentId: "sc-18y69fq-0"
})(["min-width:0px;transition:box-shadow 0.2s,min-width 0.2s,outline 0.2s;background-color:", ";", " ", ";"], function (_ref) {
  var bgColor = _ref.bgColor,
    theme = _ref.theme;
  return bgColor || theme.palette.background.paper;
}, function (_ref2) {
  var readOnly = _ref2.readOnly,
    theme = _ref2.theme,
    width = _ref2.width;
  return !readOnly && width && width < 400 && "\n    position: absolute !important;\n    min-width:400px;\n    min-height:40px;\n    max-width:800px;\n    z-index:1;\n    box-shadow:".concat(theme.shadows[8], ";\n    padding:4px;\n    border-radius:2px;\n    outline: 2px solid ").concat(theme.palette.primary.main, " !important;\n    top:0;\n    left:0;\n  ");
}, function (_ref3) {
  var readOnly = _ref3.readOnly,
    width = _ref3.width;
  return !readOnly && width && width >= 400 && "\n    width:100%;\n  ";
});
/**
 * 文本编辑器
 * @param props
 * @returns
 */
function SlateEditor(props) {
  var valueProp = props.value,
    bgColor = props.bgColor,
    width = props.width,
    fields = props.fields,
    rowIdx = props.rowIdx,
    colIdx = props.colIdx,
    editor = props.editor,
    readOnly = props.readOnly,
    other = (0, _objectWithoutProperties2.default)(props, _excluded);
  var state = (0, _reactRedux.useStore)().getState().present;
  var _useState = (0, _react.useState)(valueProp !== null && valueProp !== void 0 ? valueProp : _printOutCore.DEFAULT_CELL_VALUE),
    _useState2 = (0, _slicedToArray2.default)(_useState, 2),
    value = _useState2[0],
    setValue = _useState2[1];
  (0, _useWatchProp.default)(valueProp !== null && valueProp !== void 0 ? valueProp : _printOutCore.DEFAULT_CELL_VALUE, setValue);
  var currentSelection = (0, _react.useRef)({});
  (0, _react.useEffect)(function () {
    if (editor.selection) {
      currentSelection.current = editor.selection;
    }
  }, [editor.selection]);
  var handleChange = (0, _react.useCallback)(function (newValue) {
    if (editor.selection && !(0, _isEqual.default)(editor.selection, currentSelection.current)) {
      currentSelection.current = editor.selection;
      _constants.eventbus.emit(_constants.EDITOR_CHANGE, editor);
    }
    setValue(newValue);
  }, [editor]);
  (0, _useWatchProp.default)(readOnly, function (isReadonly) {
    if (isReadonly) {
      var prevValue = (0, _getCellValue.default)(state, rowIdx, colIdx);
      if (prevValue !== value) {
        setValue(prevValue);
      }
    }
  });
  var handleKeyDown = (0, _react.useCallback)(function (event) {
    if (event.key === 'Enter' && (0, _isCodeType.default)(editor, editor.selection.focus)) {
      event.preventDefault();
    }
  }, [editor]);
  var handleMouseUp = (0, _react.useCallback)(function () {
    (0, _autoSelectCodeBlock.default)(editor);
  }, [editor]);
  var decorate = (0, _react.useCallback)(function (_ref4) {
    var _ref5 = (0, _slicedToArray2.default)(_ref4, 2),
      node = _ref5[0],
      path = _ref5[1];
    var ranges = [];
    if (_slate.Text.isText(node)) {
      var text = node.text;
      var codeBlocks = (0, _printOutCore.findAllCodes)(text);
      codeBlocks.forEach(function (codeBlock) {
        var start = codeBlock.start;
        var tokens = (0, _helperFormulaAnalysis.genToken)(codeBlock.text, fields);
        // eslint-disable-next-line no-restricted-syntax
        var _iterator = (0, _createForOfIteratorHelper2.default)(tokens),
          _step;
        try {
          for (_iterator.s(); !(_step = _iterator.n()).done;) {
            var token = _step.value;
            if (typeof token !== 'string') {
              ranges.push({
                type: token.type,
                anchor: {
                  path: path,
                  offset: token.start + start
                },
                focus: {
                  path: path,
                  offset: token.end + start
                },
                error: token.error,
                title: token.title
              });
            }
          }
        } catch (err) {
          _iterator.e(err);
        } finally {
          _iterator.f();
        }
      });
    }
    return ranges;
  }, [fields]);
  var renderLeaf = (0, _react.useCallback)(function (leafProps) {
    return (0, _jsxRuntime.jsx)(_Leaf.default, (0, _objectSpread2.default)({}, leafProps));
  }, []);
  return (0, _jsxRuntime.jsx)(_slateReact.Slate, {
    editor: editor,
    value: value,
    onChange: handleChange,
    children: (0, _jsxRuntime.jsx)(EditableLayout, (0, _objectSpread2.default)((0, _objectSpread2.default)({}, other), {}, {
      width: width,
      decorate: decorate,
      renderLeaf: renderLeaf,
      onMouseUp: handleMouseUp,
      onKeyDown: handleKeyDown,
      bgColor: bgColor,
      style: {
        outline: '0px solid transparent'
      },
      spellCheck: true
    }))
  });
}
var _default = /*#__PURE__*/_react.default.memo(SlateEditor);
exports.default = _default;