"use strict";

var _interopRequireDefault = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = checkSubFieldInRow;
exports.getAllCodeTokensInRow = getAllCodeTokensInRow;
exports.getAllSubFieldFromToken = getAllSubFieldFromToken;
var _slicedToArray2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/slicedToArray"));
var _toConsumableArray2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/toConsumableArray"));
var _helperFormulaAnalysis = /*#__PURE__*/require("@sinoform/helper-formula-analysis");
/**
 * 获取单元格内所有的表达式
 *
 * @param value   单元格的值
 */
function getAllCodeTokensInCell(value) {
  var codes = [];
  value.forEach(function (paragraph) {
    if (paragraph.children.length > 0) {
      paragraph.children.forEach(function (item) {
        var _item$text$match;
        var matches = (_item$text$match = item.text.match(/\$\{.*?\}/g)) !== null && _item$text$match !== void 0 ? _item$text$match : [];
        codes.push.apply(codes, (0, _toConsumableArray2.default)(matches));
      });
    }
  });
  var tokens = [];
  codes.forEach(function (item) {
    return tokens.push.apply(tokens, (0, _toConsumableArray2.default)((0, _helperFormulaAnalysis.genToken)(item)));
  });
  return tokens;
}
var checkIsSubForm = function checkIsSubForm(fields, token) {
  return fields.some(function (field) {
    if (field.type === 'form' || field.type === 'modalForm') {
      return field.title === token.content;
    }
    if (field.children) {
      return checkIsSubForm(field.children, token);
    }
    return false;
  });
};
function getAllSubFieldFromToken(tokens, fields) {
  var subFormItem = tokens.filter(function (item) {
    return item.type === 'variable' && checkIsSubForm(fields, item);
  }).map(function (item) {
    return item.content;
  });
  return new Set(subFormItem);
}
/**
 * 获取当前行所有单元格公式的token信息
 *
 * @param draft   状态
 * @param cellPos 单元格位置
 * @param value   表达式
 */
function getAllCodeTokensInRow(draft, cellPos, value) {
  var _cellPos = (0, _slicedToArray2.default)(cellPos, 2),
    rowIdx = _cellPos[0],
    colIdx = _cellPos[1];
  var rows = draft.rows,
    cols = draft.cols;
  var tokens = [];
  for (var i = 0; i < cols.len; i += 1) {
    var _rows$rowIdx, _rows$rowIdx$cells, _rows$rowIdx$cells$i;
    if (colIdx === i) {
      tokens.push.apply(tokens, (0, _toConsumableArray2.default)(getAllCodeTokensInCell(value)));
    } else if ((_rows$rowIdx = rows[rowIdx]) !== null && _rows$rowIdx !== void 0 && (_rows$rowIdx$cells = _rows$rowIdx.cells) !== null && _rows$rowIdx$cells !== void 0 && (_rows$rowIdx$cells$i = _rows$rowIdx$cells[i]) !== null && _rows$rowIdx$cells$i !== void 0 && _rows$rowIdx$cells$i.value) {
      tokens.push.apply(tokens, (0, _toConsumableArray2.default)(getAllCodeTokensInCell(rows[rowIdx].cells[i].value)));
    }
  }
  return tokens;
}
/**
 * 同一行单元格的表达式中不能出现不同子表单的表单项
 * 表达式中出现了子表单选项一律视为表单项的值，不考虑其作为打印值的场景
 *
 * @param draft     状态
 * @param cellPos   单元格位置
 * @param value     表达式的值
 * @param fields    表单项集合
 */
function checkSubFieldInRow(draft, cellPos, value) {
  var fields = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  var tokens = getAllCodeTokensInRow(draft, cellPos, value);
  var set = getAllSubFieldFromToken(tokens, fields);
  return set.size > 1;
}