"use strict";

var _interopRequireDefault = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformCMToPxByConfig = exports.getCellWidth = exports.getCellHeight = exports.getCellBorderStyle = exports.checkTdIsMerged = void 0;
var _slicedToArray2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/slicedToArray"));
var _printOutCore = /*#__PURE__*/require("@sinoform/print-out-core");
/**
 * 判断一个单元格是否被合并，用于单元格渲染时的判断条件
 * 如果被合并返回true，将不再被渲染
 * 需要注意的是，合并单元格第一个位置上的单元格因为需要被渲染，所以返回false
 * @param rowIdx  单元格所在行
 * @param colIdx  单元格所在列
 * @param merges  所有的合并单元格信息
 * @returns
 */
var checkTdIsMerged = function checkTdIsMerged(rowIdx, colIdx, merges) {
  if (!merges) {
    return false;
  }
  var result = merges.find(function (_ref) {
    var _ref2 = (0, _slicedToArray2.default)(_ref, 2),
      _ref2$ = (0, _slicedToArray2.default)(_ref2[0], 2),
      sri = _ref2$[0],
      sci = _ref2$[1],
      _ref2$2 = (0, _slicedToArray2.default)(_ref2[1], 2),
      eri = _ref2$2[0],
      eci = _ref2$2[1];
    return sri <= rowIdx && eri >= rowIdx && sci <= colIdx && eci >= colIdx;
  });
  if (!result) {
    return false;
  }
  return !(rowIdx === result[0][0] && colIdx === result[0][1]);
};
exports.checkTdIsMerged = checkTdIsMerged;
var getBorderStyle = function getBorderStyle(type, color) {
  switch (type) {
    case 'thin':
      return "1px solid ".concat(color);
    case 'medium':
      return "2px solid ".concat(color);
    case 'thick':
      return "3px solid ".concat(color);
    case 'dashed':
      return "1px dashed ".concat(color);
    case 'dotted':
      return "1px dotted ".concat(color);
    default:
      return "1px solid ".concat(color);
  }
};
/**
 * 转换单元格的边框样式
 * @param border
 * @returns
 */
var getCellBorderStyle = function getCellBorderStyle() {
  var border = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var style = {};
  Object.entries(border).forEach(function (_ref3) {
    var _ref4 = (0, _slicedToArray2.default)(_ref3, 2),
      key = _ref4[0],
      val = _ref4[1];
    style[key] = getBorderStyle(val[0], val[1]);
  });
  return style;
};
/**
 * 根据打印配置获取对应的标尺线位置
 * @param page 页面配置信息
 * @param type 水平线或垂直线
 * @returns
 */
exports.getCellBorderStyle = getCellBorderStyle;
var transformCMToPxByConfig = function transformCMToPxByConfig(page) {
  var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'h';
  var _page$pageConfig = page.pageConfig,
    pageSize = _page$pageConfig.pageSize,
    pageDir = _page$pageConfig.pageDir,
    pagePaddingLeft = _page$pageConfig.pagePaddingLeft,
    pagePaddingRight = _page$pageConfig.pagePaddingRight,
    pagePaddingTop = _page$pageConfig.pagePaddingTop,
    pagePaddingBottom = _page$pageConfig.pagePaddingBottom;
  var left = parseFloat(pagePaddingLeft);
  var right = parseFloat(pagePaddingRight);
  var top = parseFloat(pagePaddingTop);
  var bottom = parseFloat(pagePaddingBottom);
  var space = type === 'h' ? top + bottom : left + right;
  return (0, _printOutCore.mmToPx)(_printOutCore.PAGE_SIZE_CONFIG[pageSize][pageDir === '横向' ? 'h' : 'v'][type === 'h' ? 1 : 0] - space);
};
/**
 * 获取单元格的高度
 * @param state
 * @param rowIdx
 * @param cell
 * @returns
 */
exports.transformCMToPxByConfig = transformCMToPxByConfig;
var getCellHeight = function getCellHeight(state, rowIdx, cell) {
  var _state$rows$rowIdx$he, _state$rows$rowIdx;
  var rowHeight = (_state$rows$rowIdx$he = (_state$rows$rowIdx = state.rows[rowIdx]) === null || _state$rows$rowIdx === void 0 ? void 0 : _state$rows$rowIdx.height) !== null && _state$rows$rowIdx$he !== void 0 ? _state$rows$rowIdx$he : _printOutCore.DEFAULT_CELL_HEIGHT;
  if (cell !== null && cell !== void 0 && cell.merge) {
    for (var i = 1; i < cell.merge[0] + 1; i += 1) {
      var _state$rows$height, _state$rows;
      rowHeight += (_state$rows$height = (_state$rows = state.rows[rowIdx + i]) === null || _state$rows === void 0 ? void 0 : _state$rows.height) !== null && _state$rows$height !== void 0 ? _state$rows$height : _printOutCore.DEFAULT_CELL_HEIGHT;
    }
  }
  return rowHeight;
};
/**
 * 获取单元格宽度
 * @param state
 * @param colIdx
 * @param cell
 * @returns
 */
exports.getCellHeight = getCellHeight;
var getCellWidth = function getCellWidth(state, colIdx, cell) {
  var _state$cols$colIdx$wi, _state$cols$colIdx;
  var cellWidth = (_state$cols$colIdx$wi = (_state$cols$colIdx = state.cols[colIdx]) === null || _state$cols$colIdx === void 0 ? void 0 : _state$cols$colIdx.width) !== null && _state$cols$colIdx$wi !== void 0 ? _state$cols$colIdx$wi : _printOutCore.DEFAULT_CELL_WIDTH;
  if (cell !== null && cell !== void 0 && cell.merge) {
    for (var i = 1; i < cell.merge[1] + 1; i += 1) {
      var _state$cols$width, _state$cols;
      cellWidth += (_state$cols$width = (_state$cols = state.cols[colIdx + i]) === null || _state$cols === void 0 ? void 0 : _state$cols.width) !== null && _state$cols$width !== void 0 ? _state$cols$width : _printOutCore.DEFAULT_CELL_WIDTH;
    }
  }
  return cellWidth;
};
exports.getCellWidth = getCellWidth;