"use strict";

var _interopRequireDefault = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = calcCellRange;
var _slicedToArray2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/slicedToArray"));
/**
 * 计算选中单元格的区域
 *
 * 如果选中区域存在合并的单元格区域超出了选中的边界，需要将选中区域拓展到新的边界，
 * 如果拓展后的边界重新触发了该规则，则递归进行计算
 *
 * @param startPos 选中区域起始位置
 * @param endPos   选中区域结束位置
 * @param merges   所有单元格的合并信息
 */
function calcCellRange(startPos, endPos) {
  var merges = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  // 按照左上至右下的顺序重新生成开始和结束节点
  var start = [Math.min(startPos[0], endPos[0]), Math.min(startPos[1], endPos[1])];
  var end = [Math.max(startPos[0], endPos[0]), Math.max(startPos[1], endPos[1])];
  // 判断这个矩形区域内有没有超出边界的合并单元格，如果存在则将单元格的边界拓展到最边缘的位置
  var _loop = function _loop(i) {
    var _loop2 = function _loop2(j) {
      var merge = merges.find(function (_ref) {
        var _ref2 = (0, _slicedToArray2.default)(_ref, 2),
          _ref2$ = (0, _slicedToArray2.default)(_ref2[0], 2),
          sri = _ref2$[0],
          sci = _ref2$[1],
          _ref2$2 = (0, _slicedToArray2.default)(_ref2[1], 2),
          eri = _ref2$2[0],
          eci = _ref2$2[1];
        return sri <= i && eri >= i && sci <= j && eci >= j;
      });
      if (merge && (merge[0][0] < start[0] || merge[0][1] < start[1] || merge[1][0] > end[0] || merge[1][1] > end[1])) {
        var newStart = [Math.min(merge[0][0], start[0]), Math.min(merge[0][1], start[1])];
        var newEnd = [Math.max(merge[1][0], end[0]), Math.max(merge[1][1], end[1])];
        return {
          v: {
            v: calcCellRange(newStart, newEnd, merges)
          }
        };
      }
    };
    for (var j = start[1]; j <= end[1]; j += 1) {
      var _ret2 = _loop2(j);
      if (typeof _ret2 === "object") return _ret2.v;
    }
  };
  for (var i = start[0]; i <= end[0]; i += 1) {
    var _ret = _loop(i);
    if (typeof _ret === "object") return _ret.v;
  }
  return [start, end];
}